import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import java.sql.*;
/**
 * Clase Model para Detalle a ser usada en JTable
 * En Crear Factura
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class DetalleModel extends AbstractTableModel  {
    private java.util.List<Detalle> lst = new ArrayList<Detalle>();
    private JTable tbl = null;
    private JComboBox<String> cboProd = null;
    private JTextField txtTotal = null;
    private JPanel contenedor = null;
    /**
     * Constructor for objects of class DetalleModel
     */
    public DetalleModel() {
        this(null,null,null);
    }
    public DetalleModel(JComboBox<String> cboP,JTextField txtT,JPanel cont) {
        cboProd = cboP;
        txtTotal = txtT;
        contenedor = cont;
    }

    // CUSTOM
    public void setJTable(JTable t) {
        tbl = t;
    }
    public java.util.List<Detalle> getDetalle() { return lst; }
    public void actualizar() {
        fireTableDataChanged();
    }
    public double getTotal() {
        double total = 0.0;
        for(Detalle d : lst) total+=d.getSubtotal();
        return total;
    }
    
    // METODOS QUE PODEMOS SOBRE-ESCRIBIR AbstractTableModel
    public Class<?> getColumnClass(int columnIndex) {
        switch(columnIndex) {
            case 0: // idp
                return Integer.class;
            case 1: // descr
                return String.class;
            case 2: // cantidad
                return Integer.class;
            case 3: // precio
                return Double.class;
            case 4: // subtotal
                return Double.class;
        }
        return  Object.class;
    }
    public String getColumnName(int column) {
        switch(column) {
            case 0: // idp
                return "Codigo";
            case 1: // descr
                return "Descripcion";
            case 2: // cantidad
                return "Cantidad";
            case 3: // precio
                return "Precio";
            case 4: // subtotal
                return "SubTotal";
        }
        return "";
    }
    public boolean isCellEditable(int rowIndex,int columnIndex) {
        if ( columnIndex == 0 || columnIndex == 1 || columnIndex == 4) return false; // no permito cambios en la columnas
        return true;
    }
    public void setValueAt(Object aValue,int rowIndex,int columnIndex) {
        Detalle c = lst.get(rowIndex);
        switch(columnIndex) {
            case 2: // cantidad
                if ( ((Integer) aValue).intValue() <= 0 ) {
                    JOptionPane.showMessageDialog(null,"Debe Indicar Cantidad","Modifico Factura", 
                        JOptionPane.ERROR_MESSAGE);
                } else {
                    c.setCantidad((Integer) aValue);
                    actualizar();
                    if ( txtTotal != null ) {
                        txtTotal.setText(""+getTotal());
                        contenedor.invalidate();
                        contenedor.validate();
                    }
                }
                break;
            case 3: // precio
                if ( ((Double) aValue).doubleValue() <= 0 ) {
                    JOptionPane.showMessageDialog(null,"El Precio no puede ser Negativo o Cero","Modifico Factura", 
                        JOptionPane.ERROR_MESSAGE);
                } else {
                    c.setPrecio((Double) aValue);
                    actualizar();
                    if ( txtTotal != null ) {
                        txtTotal.setText(""+getTotal());
                        contenedor.invalidate();
                        contenedor.validate();
                    }
                }
                break;
        }
    }
    // METODOS ABSTRACTOS OBLIGATORIOS QUE DEBEMOS IMPLEMENTAR
    public int getRowCount() {
        return lst.size(); // la cantidad de elementos que tiene lst
    }
    public int getColumnCount() {
        return 5; // porque detalle tiene 5 atributos: idp,descr,cantidad,precio,subtotal
    }
    public Object getValueAt(int row, int column) {
        Detalle c = lst.get(row);
        switch(column) {
            case 0: // idp
                return c.getProducto().getIdp();
            case 1: // descr
                return c.getProducto().getDescr();
            case 2: // cantidad
                return c.getCantidad();
            case 3: // precio
                return c.getPrecio();
            case 4:
                return c.getSubtotal();
        }
        return  null;
    }
    // CUSTOM
    public void agregar() {
        System.out.println("DetalleModel presiono Insert!!");
        if ( cboProd == null ) System.out.println("DetalleModel:Error!, cboProd no puede ser null!");
        String sprod = (String) cboProd.getSelectedItem();
        System.out.println("sprod="+sprod);
        int ppos = sprod.indexOf("(");
        String sdescr = sprod.substring(0,ppos).trim();
        int ppos2 = sprod.indexOf(")",ppos+1);
        int idp = new Integer(sprod.substring(ppos+1,ppos2));
        ppos = sprod.lastIndexOf("(");
        ppos2 = sprod.lastIndexOf(")");
        System.out.println("ppos=" + ppos + " ppos2="+ppos2);
        double precio = new Double(sprod.substring(ppos+1,ppos2));
        Producto c = new Producto(idp,sdescr,precio);
        Detalle d = new Detalle(c,1,precio);
        boolean repetido = false;
        for(Detalle dd : lst) {
            if ( dd.getProducto().getIdp() == idp ) {
                dd.setCantidad(dd.getCantidad()+1);
                repetido = true;
                break;
            }
        }
        // agrego nuevo producto a la lista
        if (!repetido) lst.add(d);
        actualizar(); // informo a la JTable que he cambiado los datos!
        if ( txtTotal != null ) {
            System.out.println("agregar() seteo total a " + getTotal());
            txtTotal.setText(""+getTotal());
            contenedor.invalidate();
            contenedor.validate();
        }
    }
    public void borrar() {
        if ( tbl != null ) {
            if ( tbl.getSelectedRow() != -1 ) {
                // pedir confirmacion!
                if ( JOptionPane.showOptionDialog(null,"¿Borro a la linea de Factura Seleccionada?",
                    "Borro Detalle Factura", JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE,
                        null,new String[] { "Si", "No" } , "Si") == 0 ) {
                    lst.remove(tbl.getSelectedRow());
                    actualizar(); // informo a la JTable que he cambiado los datos!
                    if ( txtTotal != null ) {
                        System.out.println("borrar() seteo total a " + getTotal());
                        txtTotal.setText(""+getTotal());
                        contenedor.invalidate();
                        contenedor.validate();
                    }
                }
            } else System.out.println("Debe Seleccionar linea de Detalle a Borrar");
        } else System.out.println("Debe Indicar la JTable asociada a DetalleModel");
    }
   
}
